/* file_multi.c: read multiple types of files as text data. */

/*
All code included in versions up to and including 2.09:
   Copyright (C) 2001 Massachusetts Institute of Technology.

All modifications subsequent to version 2.09 are copyright Bradley
Rhodes or their respective authors.

Developed by Bradley Rhodes at the Media Laboratory, MIT, Cambridge,
Massachusetts, with support from British Telecom and Merrill Lynch.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.  For commercial licensing under other
terms, please consult the MIT Technology Licensing Office.

This program may be subject to the following US and/or foreign
patents (pending): "Method and Apparatus for Automated,
Context-Dependent Retrieval of Information," MIT Case No. 7870TS. If
any of these patents are granted, royalty-free license to use this
and derivative programs under the GNU General Public License are
hereby granted.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.  */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_LIBZ
#include <zlib.h>
#endif /* HAVE_LIBZ */

#include "file_multi.h"

#ifdef HAVE_LIBZ
size_t gz_read(struct FILE_multi *file, char *buf, size_t len)
{
  return gzread((gzFile) file->data, buf, len);
}

int gz_seek(struct FILE_multi *file, long offset, int whence)
{
  return gzseek((gzFile) file->data, offset, whence);
}

long gz_tell(struct FILE_multi *file)
{
  return gztell((gzFile) file->data);
}

int gz_error(struct FILE_multi *file)
{
  int errnum = 0;
  const char *errstr;

  errstr =  gzerror((gzFile) file->data, &errnum);

  return errnum;
}

int gz_eof(struct FILE_multi *file)
{
  return gzeof((gzFile) file->data);
}

void gz_close(struct FILE_multi *file)
{
  gzclose((gzFile) file->data);
}
#else /* !HAVE_LIBZ */
size_t stdio_read(struct FILE_multi *file, char *buf, size_t len)
{
  return fread(buf, sizeof(char), len, (FILE*) file->data);
}

int stdio_seek(struct FILE_multi *file, long offset, int whence)
{
  return fseek((FILE*) file->data, offset, whence);
}

long stdio_tell(struct FILE_multi *file)
{
  return ftell((FILE*) file->data);
}

int stdio_error(struct FILE_multi *file)
{
  return ferror((FILE*) file->data);
}

int stdio_eof(struct FILE_multi *file)
{
  return feof((FILE*) file->data);
}

void stdio_close(struct FILE_multi *file)
{
  fclose((FILE*) file->data);
}
#endif /* HAVE_LIBZ */

/* Open file. */
struct FILE_multi *file_multi_open(const char *path, const char *mode)
{
  struct FILE_multi *file = NULL;
#ifdef HAVE_LIBZ
  gzFile *gz_file;
#else /* !HAVE_LIBZ */
  FILE *stdio_file;
#endif /* HAVE_LIBZ */

#ifdef HAVE_LIBZ
  gz_file = gzopen(path, mode);
  if (gz_file) {
    file = malloc(sizeof(struct FILE_multi));
    file->read = &gz_read;
    file->seek = &gz_seek;
    file->tell = &gz_tell;
    file->error = &gz_error;
    file->eof = &gz_eof;
    file->close = &gz_close;
    file->data = (void*) gz_file;
  }
#else /* ! HAVE_LIBZ */
  stdio_file = fopen(path, mode);
  if (stdio_file) {
    file = malloc(sizeof(struct FILE_multi));
    file->data = (void*) stdio_file;
    file->read = &stdio_read;
    file->seek = &stdio_seek;
    file->tell = &stdio_tell;
    file->error = &stdio_error;
    file->eof = &stdio_eof;
    file->close = &stdio_close;
    file->data = (void*) stdio_file;
  }
#endif /* HAVE_LIBZ */

  return file;
}

/* Close file freeing resources. */
void file_multi_close (struct FILE_multi *file)
{
  file->close(file);
  free(file);
}
